#include "nsocv1/soc.h"
#include <stdint.h>
#include "akida_dma.h"
#include "elog.h"
#include "systick.h"

#define BOOT_STATE_OFFSET 12
#define BOOT_STATE_MASK 0x1

#define COLD_BOOT 0
#define WARM_BOOT 1

__WEAK int mem_init(void);
#ifndef CONFIG_LOG_SOC_LEVEL
#define CONFIG_LOG_SOC_LEVEL CONFIG_LOG_DEFAULT_LEVEL
#endif
DECLARE_LOG_MODULE(SOC, CONFIG_LOG_SOC_LEVEL);
static inline uint8_t read_boot_state(void) {
    return (SCRST->ARMSTATE >> BOOT_STATE_OFFSET) & BOOT_STATE_MASK;
}

static inline void set_boot_state(uint8_t state) {
    SCRST->ARMSTATE &= ~(BOOT_STATE_MASK << BOOT_STATE_OFFSET);
    SCRST->ARMSTATE |= (state << BOOT_STATE_OFFSET);
}

int gpio_init() {
    *(uint32_t*)(GPIO_BASE + 0x20) = 0;
    *(uint32_t*)(GPIO_BASE + 0x24) = 0;
    *(uint32_t*)(GPIO_BASE + 0x28) = 0;
    *(uint32_t*)(GPIO_BASE + 0x2c) = 0;

    *(uint32_t*)(GPIO_BASE + 0x10) = 0xff;
    *(uint32_t*)(GPIO_BASE + 0x14) = 0xff;
    *(uint32_t*)(GPIO_BASE + 0x18) = 0xff;
    *(uint32_t*)(GPIO_BASE + 0x1c) = 0xff;

    *(uint32_t*)(GPIO_BASE) = 0xff;
    *(uint32_t*)(GPIO_BASE + 4) = 0xff;
    return 0;
}

void gpio_set_segments(uint8_t number) {
    const uint16_t left[] = {
        0xe07f, 0xfcff, 0xd27f, 0xd87f, 0xccff, 0xc97f, 0xc17f, 0xfc7f,
        0xc07f, 0xc87f, 0x047f, 0x01ff, 0x237f, 0x10ff, 0xc37f, 0xc77f,
    };

    const uint16_t right[] = {
        0xffc0, /* 0 */
        0xfff9, /* 1 */
        0xffa4, 0xffb0, 0xff99, 0xff92, 0xff82, 0xfff8, 0xff80,
        0xff98, 0xff88, 0xff83, 0xffc6, 0xffa1, 0xff86, 0xff8e,

    };
    uint16_t data = left[number / 16] & right[number % 16];

    *(uint8_t*)(GPIO_BASE + 4) = data >> 8;
    *(uint8_t*)(GPIO_BASE) = data & 0xff;
}

void soc_init() {
    /* Initialize soc level drivers with no driver interfaces
     */
    gpio_init();
    gpio_set_segments(0);
    if (crg11_init(0) != 0) {
        SOC_LOGE("Error initializing CRG11");
    }
    gpio_set_segments(1);

    for (volatile int i = 0; i < 10000; i++)
        ;
    // Raise and error if PLL not locked
    if ((CRG11->CRRDY & 1) == 0) {
        // Error
        gpio_set_segments(0xe1);
        while (1)
            ;
    }

    // Reduce APB clock to 33Mhz (CLK7 DIV4 600/18)
    CRG11->CLK[7].CRDM = CRG11_CRDM_DIVMODE_18;
    CRG11->CRCDC = 1;

    systick_init();
    if (!(read_boot_state() && WARM_BOOT)) {
        int ret = mem_init();
        if (ret) {
            gpio_set_segments(0xe0 | -ret);
            while (1)
                ;
        }
        set_boot_state(WARM_BOOT);
    }

    int revision = (SCRST->CHIPINFO & 0x7);
    gpio_set_segments(0xd0 | revision);
    SOC_LOGI("SoC revision: %d", revision);

    if (akida_hdmac_init()) {
        SOC_LOGI("DMA Initialization failed");
    } else {
        SOC_LOGI("DMA Initialization successfull");
    }
}
