#include "i2c_bus.h"
#include <string.h>
#include "bsp_error.h"
#include "elog.h"

int i2c_read(struct i2c_device* drv, uint8_t* buf, int len, i2c_callback_t cb, void* cb_param) {
    if (drv->bus && drv->bus->ops->read) {
        return drv->bus->ops->read(drv, buf, len, cb, cb_param);
    }
    return ENOT_IMPLEMENTED;
}

int i2c_write(struct i2c_device* drv, uint8_t* buf, int len, i2c_callback_t cb, void* cb_param) {
    if (drv->bus && drv->bus->ops->write) {
        return drv->bus->ops->write(drv, buf, len, cb, cb_param);
    }
    return ENOT_IMPLEMENTED;
}

int i2c_read_reg(struct i2c_device* drv, uint8_t reg, uint8_t* buf, int len) {
    if (drv->bus && drv->bus->ops->read_reg) {
        return drv->bus->ops->read_reg(drv, reg, buf, len);
    }
    return ENOT_IMPLEMENTED;
}

int i2c_write_reg(struct i2c_device* drv, uint8_t reg, uint8_t* buf, int len) {
    if (drv->bus && drv->bus->ops->write_reg) {
        return drv->bus->ops->write_reg(drv, reg, buf, len);
    }
    return ENOT_IMPLEMENTED;
}

int i2c_attach(struct i2c_bus_driver* bus, struct i2c_device* dev, struct driver_common* intf,
               struct config_item* configs) {
    int ret = 0;

    /* Add to list */
    dev->next = bus->devices;
    bus->devices = dev;

    /* Set underlying bus */
    dev->bus = bus;

    I2C_LOGI("Attach i2c device %s to bus %s", dev->name, bus->driver.reg->name);

    if (intf) {
        /* Copy the init, deinit pointers, as well as the ops and priv pointer */
        memcpy(&dev->driver, intf, sizeof(struct driver_common) + 2 * sizeof(void*));
    }

    dev->driver.config_items = configs;
    memcpy(dev->driver.config_items, intf->config_items,
           intf->config_items_count * sizeof(*configs));

    if (bus->ops->attach) {
        ret = bus->ops->attach(bus, dev);
        if (ret)
            goto error;
    }
    if (dev->driver.init) {
        ret = dev->driver.init(&dev->driver, dev);
    }
    if (dev->driver.read_config)
        dev->driver.read_config(&dev->driver);
error:
    return ret;
}
