#include <stdarg.h>
#include <stddef.h>
#include <stdint.h>
#include <string.h>

#ifndef __CLI_CMD_H__
#define __CLI_CMD_H__

struct cli_cmd_ctx;
struct cli_ctx;

struct cli_cmd_handler {
    const char* name;
    int (*handler)(int argc, char** argv, struct cli_cmd_ctx* ctx);
};

#define TTY_STATE_NORMAL 0
#define TTY_STATE_ESCAPE 1
#define TTY_STATE_ESCAPE_2 2

struct cli_ctx {
    int (*vprintf)(struct cli_ctx*, const char* fmt, va_list args);
    int (*get_char)(struct cli_ctx*);
    void (*puts)(struct cli_ctx*, char* s);
    void (*put)(struct cli_ctx*, int c);
    int (*write)(struct cli_ctx*, uint8_t* buf, int size);
    char buffer[256];
    int cmd_idx;
    int state;
    int bin_mode;
};

struct cli_cmd_ctx {
    struct cli_ctx* cli;
    struct cli_cmd_handler* h;
};

struct cli_channel_handler {
    char* name;
    struct cli_ctx* ctx;
    int (*init)(struct cli_channel_handler*);
    void* priv;
};

#ifdef DISABLE_CLI
#define DECLARE_CMD(_name_, _handler_)
#define DECLARE_TEST_CMD(_name_, _handler_)
#define DECLARE_DEBUG_CMD(_name_, _handler_)
#define DECLARE_CONFIG_CMD(_name_, _handler_)
#define DECLARE_CLI_CHANNEL(_name_, _ctx_, _init_, _priv_)
#else
#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
#define DECLARE_CMD(_name_, _handler_)                                            \
    struct cli_cmd_handler __attribute__((section("CMDS"))) __attribute__((used)) \
        __cmd_handler_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_TEST_CMD(_name_, _handler_)                                            \
    struct cli_cmd_handler __attribute__((section("TEST_CMDS"))) __attribute__((used)) \
        __cmd_handler_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_DEBUG_CMD(_name_, _handler_)                                            \
    struct cli_cmd_handler __attribute__((section("DEBUG_CMDS"))) __attribute__((used)) \
        __cmd_handler_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_CONFIG_CMD(_name_, _handler_)                                            \
    struct cli_cmd_handler __attribute__((section("CONFIG_CMDS"))) __attribute__((used)) \
        __cmd_handler_config_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_CLI_CHANNEL(_name_, _ctx_, _init_, _priv_)                                    \
    struct cli_channel_handler __attribute__((section("CLI_CHANNELS"))) __attribute__((used)) \
        __cmd_channel_handler_##_name_ = {                                                    \
            .name = #_name_, .ctx = _ctx_, .init = _init_, .priv = _priv_};

#elif defined(__GNUC__)
#define DECLARE_CMD(_name_, _handler_)                               \
    struct cli_cmd_handler __attribute__((section(".cmd." #_name_))) \
        __cmd_handler_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_TEST_CMD(_name_, _handler_)                               \
    struct cli_cmd_handler __attribute__((section(".test.cmd." #_name_))) \
        __cmd_handler_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_DEBUG_CMD(_name_, _handler_)                               \
    struct cli_cmd_handler __attribute__((section(".debug.cmd." #_name_))) \
        __cmd_handler_debug_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_CONFIG_CMD(_name_, _handler_)                               \
    struct cli_cmd_handler __attribute__((section(".config.cmd." #_name_))) \
        __cmd_handler_config_##_name_ = {.name = #_name_, .handler = _handler_};

#define DECLARE_CLI_CHANNEL(_name_, _ctx_, _init_, _priv_)                       \
    struct cli_channel_handler __attribute__((section(".cli.channel." #_name_))) \
        __cmd_channel_handler_##_name_ = {                                       \
            .name = #_name_, .ctx = _ctx_, .init = _init_, .priv = _priv_};
#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif
int cli_printf(struct cli_cmd_ctx* ctx, const char* fmt, ...);
void cli_exec(char* cmd, struct cli_ctx* ctx);
void cli_dump_mem(struct cli_cmd_ctx* ctx, int address, int buffer, int len, int size);
void cli_handle_char(struct cli_ctx* ctx, char c);
void cli_set_bin_mode(struct cli_ctx* ctx, int bin_mode);
void cli_init();
void cli_poll();
#ifdef __cplusplus
}
#endif

#endif /* __CLI_CMD_H__ */
