#ifndef __COMMON_H__
#define __COMMON_H__

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "bsp_error.h"
#include "device.h"
#if defined(__CC_ARM)
#include "stdarg.h"
#endif
#include "elog.h"
#include "systick.h"

#define REG_WRITE_MASK(REG, SETVAL, CLEARMASK) (REG |= ((REG &= ~(CLEARMASK)) | (SETVAL)))
#define REG_WRITE(REG, SETVAL) (REG = (SETVAL))
#define REG_CLEAR_BIT_FIELD(REG, CLEARBITMASK) (REG &= (~CLEARBITMASK))
#define REG_READ_MASK(REG, MASK) (REG & (MASK))
#define REG_READ(REG) (REG)

/*
 * Bit Operations
 */
#define BC_BIT(x) (1 << (x))
#define BC_GET_BIT(x, n) (((x) >> (n)) && 0x1)
#define DEFAULTU32BITMASK 0xFFFFFFFF

#define BCFW_VALIDATE_NULL(POINTER) \
    if (NULL == POINTER) {          \
        return ENULL_POINTER;       \
    }

#define BCFW_VALIDATE_RETURN(retval) \
    {                                \
        if (SUCCESS != retval)       \
            return retval;           \
    }

/**
 * Disable interrupts
 * @return current interrupt state.
 */
uint32_t interrupt_lock();
/**
 * Restore interrupt state
 *
 * @param flag interrupt state as returned by @ref interrupt_lock()
 */
void interrupt_unlock(uint32_t flag);

/**
 * This method performs the bsp initialization. should be called first
 * in application main()
 */
void bsp_init(void);
#endif /*__COMMON_H__*/
